SystemOrganization addCategory: #'EjerciciosPoo-TusLibros'!

TestCase subclass: #TusLibrosTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'GrossodelSel 6/13/2012 16:15'!
aBookIsbnOutOfEditorial

	^ 'this book isbn is not in the Editorial'.! !

!TusLibrosTest methodsFor: 'editorial-books' stamp: 'GrossodelSel 6/13/2012 16:32'!
bookIsbn1
	
	^ 'bookIsbn1'.! !

!TusLibrosTest methodsFor: 'editorial-books' stamp: 'GrossodelSel 6/13/2012 16:48'!
bookIsbn2
	
	^ 'bookIsbn2'.! !

!TusLibrosTest methodsFor: 'editorial-books' stamp: 'GrossodelSel 6/13/2012 16:48'!
bookIsbn3
	
	^ 'bookIsbn3'.! !

!TusLibrosTest methodsFor: 'editorial-books' stamp: 'GrossodelSel 6/13/2012 16:48'!
bookIsbnsOfEditorial

	^ Set with: self bookIsbn1 with: self bookIsbn2 with: self bookIsbn3.! !

!TusLibrosTest methodsFor: 'error' stamp: 'GrossodelSel 6/13/2012 16:06'!
canNotAddAnInvalidQuantityOfBookIsbnsError

	^ Error.! !

!TusLibrosTest methodsFor: 'error' stamp: 'GrossodelSel 6/13/2012 16:24'!
canNotAddToCartABookIsbnThatIsNotFromTheEditorialError
	
	^ Error.
! !

!TusLibrosTest methodsFor: 'cart' stamp: 'GrossodelSel 6/13/2012 16:22'!
newCart

	^ Cart withEditorialBookIsbns: self bookIsbnsOfEditorial.! !

!TusLibrosTest methodsFor: 'cart' stamp: 'GrossodelSel 6/13/2012 17:08'!
newCartWithOneCopyOfEachBookIsbn

	| cart |
	cart := self newCart.
	
	self bookIsbnsOfEditorial do:
		[ :aBookIsbn | cart add: aBookIsbn quantity: 1 ].
	
	^ cart.! !

!TusLibrosTest methodsFor: 'test-cart' stamp: 'GrossodelSel 6/13/2012 15:54'!
test01CartShouldBeEmptyAfterCreation

	self assert: self newCart isEmpty.! !

!TusLibrosTest methodsFor: 'test-cart' stamp: 'GrossodelSel 6/13/2012 17:24'!
test02CartShouldNotBeEmptyAfterAddingABookIsbnOfTheEditorial
	
	| cart |
	cart := self newCart.
	
	cart add: (self bookIsbn1) quantity: 1.
	
	self deny: cart isEmpty.! !

!TusLibrosTest methodsFor: 'test-cart' stamp: 'GrossodelSel 6/13/2012 16:07'!
test03CanNotAddAnInvalidQuantityOfBookIsbnsToCart
	
	self should: [ self newCart add: 'aBookISBN' quantity: -1 ] raise: self canNotAddAnInvalidQuantityOfBookIsbnsError.
! !

!TusLibrosTest methodsFor: 'test-cart' stamp: 'GrossodelSel 6/13/2012 16:24'!
test04CanNotAddToCartABookIsbnThatIsNotFromTheEditorial
	
	self should: [ self newCart add: (self aBookIsbnOutOfEditorial) quantity: 1 ] raise: self canNotAddToCartABookIsbnThatIsNotFromTheEditorialError.
! !

!TusLibrosTest methodsFor: 'test-cart' stamp: 'GrossodelSel 6/13/2012 16:51'!
test05CartShouldContainTheSameAmountOfAGivenBookIsbnThatWasAdded
	
	| cart |
	cart := self newCart.
	
	cart add: self bookIsbn1 quantity: 2.
	self assert: (cart quantityOf: self bookIsbn1) = 2.
	
	cart add: self bookIsbn1 quantity: 3.
	self assert: (cart quantityOf: self bookIsbn1) = 5.
	
	self assert: (cart quantityOf: self bookIsbn2) = 0.! !

!TusLibrosTest methodsFor: 'test-cart' stamp: 'GrossodelSel 6/13/2012 17:07'!
test06CartCanBeEnumeratedUsingDo

	| initialSetOfBookIsbns enumeratedSetOfBookIsbns |
	
	initialSetOfBookIsbns := self bookIsbnsOfEditorial.
	enumeratedSetOfBookIsbns := Set new.
	
	(self newCartWithOneCopyOfEachBookIsbn) do: [ :aBookIsbn |
		enumeratedSetOfBookIsbns add: aBookIsbn ].
	
	self assert: initialSetOfBookIsbns = enumeratedSetOfBookIsbns.
! !

Object subclass: #Cart
	instanceVariableNames: 'bookIsbns editorialBookIsbns'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Cart class methodsFor: 'as yet unclassified' stamp: 'GrossodelSel 6/13/2012 16:19'!
withEditorialBookIsbns: aSetOfBookIsbns

	^ Cart new initializeWithEditorialBookIsbns: aSetOfBookIsbns.! !

!Cart methodsFor: 'adding' stamp: 'GrossodelSel 6/13/2012 16:29'!
add: aBookIsbn quantity: aQuantity

	(aQuantity <= 0) ifTrue:
		[ self error: self invalidQuantityErrorMessage ].
	
	(editorialBookIsbns includes: aBookIsbn) ifFalse:
		[ self error: self invalidBookIsbnErrorMessage ].
	
	bookIsbns add: aBookIsbn withOccurrences: aQuantity.! !

!Cart methodsFor: 'enumerating' stamp: 'GrossodelSel 6/13/2012 17:10'!
do: aBlock

	bookIsbns valuesAndCounts keysDo: aBlock.! !

!Cart methodsFor: 'initializing' stamp: 'GrossodelSel 6/13/2012 16:17'!
initializeWithEditorialBookIsbns: aSetOfBookIsbns

	bookIsbns := Bag new.
	editorialBookIsbns := aSetOfBookIsbns.! !

!Cart methodsFor: 'error' stamp: 'GrossodelSel 6/13/2012 16:30'!
invalidBookIsbnErrorMessage

	^ 'the specified book isbn must belong to the editorial '.! !

!Cart methodsFor: 'error' stamp: 'GrossodelSel 6/13/2012 16:04'!
invalidQuantityErrorMessage

	^ 'the specified quantity must be greater than zero'.! !

!Cart methodsFor: 'testing' stamp: 'GrossodelSel 6/13/2012 15:43'!
isEmpty

	^ bookIsbns isEmpty.! !

!Cart methodsFor: 'accessing' stamp: 'GrossodelSel 6/13/2012 16:53'!
quantityOf: aBookIsbn 

	^ bookIsbns occurrencesOf: aBookIsbn.! !
